# Coding Style

All code generated by the SNAC project will be written in one of the following languages.

* PHP 7 (preferred)
* PHP 5
* Java
* XSLT


## Coding Style Specifications

Source code must match the following style guidelines:

* 4-space tabs with literal spaces
* Maximum line-length of 100 characters
* Variables and Class names follow standard camel casing syntax, with descriptive names
    * Class names start with upper-case letters
    * Variable and field names start with lower-clase letters
    * No underscores allowed in variable names

## Internal Documentation of Code

All code will be internally-documented using [Javadoc](http://www.oracle.com/technetwork/java/javase/documentation/index-137868.html) style documentation, which has been ported to PHP as [phpdoc](http://www.phpdoc.org/docs/latest/guides/docblocks.html).  Tools to generate documentation from the code is also available for [Java](http://www.oracle.com/technetwork/java/javase/documentation/index-jsp-135444.html) and [PHP](http://www.phpdoc.org/).

* All files, regardless of language,  must have javadoc-style documentation with author attribution, definition of the file, and short-text of the code license, as defined below:
    
    ```php
    /**
     * File Description Headline
     *
     * Paragraphs describing the file
     * 
     * License:
     * ....
     *
     * @author Robbie Hott
     * @license http://opensource.org/licenses/BSD-3-Clause BSD 3-Clause
     * @copyright 2015 the Rector and Visitors of the University of Virginia, and the Regents of the University of California
     */
    ```

* All classes, fields, methods, and function definitions must include documentation, as shown below:

    ```php
    /**
     * Name Reconciliation Engine Main Class
     *
     * This class provides the meat of the reconciliation engine. To run the
     * reconciliation engine, create an instance of this class and call the
     * reconcile method.
     *
     * @author Robbie Hott
     */
    class ReconciliationEngine {
        ...

        
        /**
         * Main reconciliation function
         *
         * This function does the reconciliation and returns the top identity from
         * the engine.  Other top identities and their corresponding score vectors
         * may be obtained by other functions within this class.  
         *
         * @param identity $identity The identity to be searched. This identity 
         * must be in the proper form 
         * @return identity The top identity by the reconciliation
         * engine
         */
        public function reconcile($identity) {
            ...
    ```
